#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from datetime import timedelta

from marshmallow import fields

from polyaxon.env_vars.keys import POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK
from polyaxon.schemas.api.log_handler import LogHandlerSchema
from polyaxon.schemas.base import BaseConfig, BaseSchema
from polyaxon.utils.tz_utils import now


class CliConfigurationSchema(BaseSchema):
    current_version = fields.Str(allow_none=True)
    installation = fields.Dict(allow_none=True)
    compatibility = fields.Dict(allow_none=True)
    log_handler = fields.Nested(LogHandlerSchema, allow_none=True)
    last_check = fields.DateTime(allow_none=True)

    @staticmethod
    def schema_config():
        return CliConfigurationConfig


class CliConfigurationConfig(BaseConfig):
    SCHEMA = CliConfigurationSchema
    IDENTIFIER = "cli"
    INTERVAL = 30 * 60

    def __init__(
        self,
        current_version=None,
        installation=None,
        compatibility=None,
        log_handler=None,
        last_check=None,
    ):
        self.current_version = current_version
        self.installation = installation
        self.compatibility = compatibility
        self.log_handler = log_handler
        self.last_check = self.get_last_check(last_check)

    def get_interval(self):
        return os.environ.get(POLYAXON_KEYS_INTERVALS_COMPATIBILITY_CHECK, self.INTERVAL)

    @classmethod
    def get_last_check(cls, last_check):
        return last_check or now() - timedelta(cls.INTERVAL)

    @property
    def min_version(self):
        if not self.compatibility or "cli" not in self.compatibility:
            return None
        cli_version = self.compatibility["cli"] or {}
        return cli_version.get("min")

    @property
    def latest_version(self):
        if not self.compatibility or "cli" not in self.compatibility:
            return None
        cli_version = self.compatibility["cli"] or {}
        return cli_version.get("latest")

    def should_check(self):
        interval = self.get_interval()
        if interval == -1:
            return False
        if (now() - self.last_check).total_seconds() > interval:
            return True
        if self.current_version is None or self.min_version is None:
            return True

        return False



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.exceptions import HTTP_ERROR_MESSAGES_MAPPING
from polyaxon.utils.formatting import Printer


def handle_cli_error(e, message=None, sys_exit: bool = False):
    if message:
        Printer.print_error(message)
    if hasattr(e, "status"):
        if e.status not in [404, 401, 403]:
            Printer.print_error("Error message: {}.".format(e))
        Printer.print_error(
            HTTP_ERROR_MESSAGES_MAPPING.get(e.status), sys_exit=sys_exit
        )
    else:
        Printer.print_error("Error message: {}.".format(e), sys_exit=sys_exit)



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest.mock import patch

import pytest

from tests.utils import BaseTestCase

from polyaxon.polyflow.matrix import V1GridSearch
from polyaxon.polytune.search_managers.grid_search.manager import GridSearchManager


@pytest.mark.polytune_mark
class TestGridSearch(BaseTestCase):
    def test_grid_search_config(self):
        assert GridSearchManager.CONFIG == V1GridSearch

    def test_get_suggestions(self):
        config = V1GridSearch.from_dict(
            {
                "concurrency": 2,
                "numRuns": 10,
                "params": {"feature": {"kind": "choice", "value": [1, 2, 3]}},
            }
        )
        assert len(GridSearchManager(config).get_suggestions()) == 3

        config = V1GridSearch.from_dict(
            {
                "concurrency": 2,
                "numRuns": 10,
                "params": {
                    "feature1": {"kind": "choice", "value": [1, 2, 3]},
                    "feature2": {"kind": "linspace", "value": [1, 2, 5]},
                    "feature3": {"kind": "range", "value": [1, 5, 1]},
                },
            }
        )
        assert len(GridSearchManager(config).get_suggestions()) == 10

    def test_get_suggestions_calls_to_numpy(self):
        config = V1GridSearch.from_dict(
            {
                "concurrency": 2,
                "numRuns": 10,
                "params": {"feature": {"kind": "choice", "value": [1, 2, 3]}},
            }
        )
        with patch(
            "polyaxon.polytune.search_managers.grid_search.manager.to_numpy"
        ) as to_numpy_mock:
            GridSearchManager(config).get_suggestions()

        assert to_numpy_mock.call_count == 1

        config = V1GridSearch.from_dict(
            {
                "concurrency": 2,
                "params": {
                    "feature1": {"kind": "choice", "value": [1, 2, 3]},
                    "feature2": {"kind": "logspace", "value": "0.01:0.1:5"},
                },
            }
        )
        with patch(
            "polyaxon.polytune.search_managers.grid_search.manager.to_numpy"
        ) as to_numpy_mock:
            GridSearchManager(config).get_suggestions()

        assert to_numpy_mock.call_count == 2



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import polyaxon_sdk


class V1MatrixKind(polyaxon_sdk.V1MatrixKind):
    CHOICES = (
        (polyaxon_sdk.V1MatrixKind.RANDOM, polyaxon_sdk.V1MatrixKind.RANDOM),
        (polyaxon_sdk.V1MatrixKind.GRID, polyaxon_sdk.V1MatrixKind.GRID),
        (polyaxon_sdk.V1MatrixKind.HYPERBAND, polyaxon_sdk.V1MatrixKind.HYPERBAND),
        (polyaxon_sdk.V1MatrixKind.BAYES, polyaxon_sdk.V1MatrixKind.BAYES),
        (polyaxon_sdk.V1MatrixKind.HYPEROPT, polyaxon_sdk.V1MatrixKind.HYPEROPT),
        (polyaxon_sdk.V1MatrixKind.ITERATIVE, polyaxon_sdk.V1MatrixKind.ITERATIVE),
        (polyaxon_sdk.V1MatrixKind.MAPPING, polyaxon_sdk.V1MatrixKind.MAPPING),
    )



#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from polyaxon.schemas.fields.dict_or_str import DictOrStr  # noqa
from polyaxon.schemas.fields.float_or_str import FloatOrStr  # noqa
from polyaxon.schemas.fields.indexed_dict import IndexedDict  # noqa
from polyaxon.schemas.fields.int_or_str import IntOrStr  # noqa
from polyaxon.schemas.fields.obj_list_obj import ObjectOrListObject  # noqa
from polyaxon.schemas.fields.str_or_fct import StrOrFct  # noqa
from polyaxon.schemas.fields.tensor import Tensor  # noqa
from polyaxon.schemas.fields.uuids import UUID  # noqa
